@@ -961,7 +961,7 @@ namespace Jrd {
 
 				// Establish context
 				// Need real attachment in order to make classic mode happy
-				ClumpletWriter writer(ClumpletReader::Tagged, MAX_DPB_SIZE, isc_dpb_version1);
+				ClumpletWriter writer(ClumpletReader::dpbList, MAX_DPB_SIZE);
 				writer.insertString(isc_dpb_user_name, "SYSDBA");
 				writer.insertByte(isc_dpb_no_db_triggers, TRUE);
 
@@ -986,7 +986,7 @@ namespace Jrd {
 				releaseGuard.leave();
 
 				ThreadContextHolder tdbb(att->att_database, att, &status_vector);
-				tdbb->tdbb_quantum = SWEEP_QUANTUM;
+				tdbb->markAsSweeper();
 
 				DatabaseContextHolder dbHolder(tdbb);
 
@@ -1024,9 +1024,7 @@ namespace Jrd {
 
 						// scheduling
 						if (--tdbb->tdbb_quantum < 0)
-						{
-							JRD_reschedule(tdbb, SWEEP_QUANTUM, true);
-						}
+							JRD_reschedule(tdbb, true);
 
 						// nbackup state check
 						BackupManager::StateReadGuard::lock(tdbb, 1);
