@@ -134,13 +134,20 @@ MonitoringData::MonitoringData(const Database* dbb)
 
 MonitoringData::~MonitoringData()
 {
-	Guard guard(this);
+	m_sharedMemory->mutexLock();
 
-	if (m_sharedMemory->getHeader() &&
-		m_sharedMemory->getHeader()->used == alignOffset(sizeof(Header)))
+	try
 	{
-		m_sharedMemory->removeMapFile();
+		if (m_sharedMemory->getHeader() &&
+			m_sharedMemory->getHeader()->used == alignOffset(sizeof(Header)))
+		{
+			m_sharedMemory->removeMapFile();
+		}
 	}
+	catch (const Exception&)
+	{} // no-op
+
+	m_sharedMemory->mutexUnlock();
 }
 
 
@@ -1380,6 +1387,8 @@ void Monitoring::publishAttachment(thread_db* tdbb)
 
 	MonitoringData::Guard guard(dbb->dbb_monitoring_data);
 	dbb->dbb_monitoring_data->setup(attachment->att_attachment_id, user_name.c_str());
+
+	attachment->att_flags |= ATT_monitor_init;
 }
 
 
@@ -1388,9 +1397,14 @@ void Monitoring::cleanupAttachment(thread_db* tdbb)
 	Database* const dbb = tdbb->getDatabase();
 	Attachment* const attachment = tdbb->getAttachment();
 
-	if (dbb->dbb_monitoring_data)
+	if (attachment->att_flags & ATT_monitor_init)
 	{
-		MonitoringData::Guard guard(dbb->dbb_monitoring_data);
-		dbb->dbb_monitoring_data->cleanup(attachment->att_attachment_id);
+		attachment->att_flags &= ~ATT_monitor_init;
+
+		if (dbb->dbb_monitoring_data)
+		{
+			MonitoringData::Guard guard(dbb->dbb_monitoring_data);
+			dbb->dbb_monitoring_data->cleanup(attachment->att_attachment_id);
+		}
 	}
 }
