@@ -241,33 +241,23 @@ private:
 
 	pollfd* getPollFd(int n)
 	{
-		pollfd* const end = slct_poll.end();
-		for (pollfd* pf = slct_poll.begin(); pf < end; ++pf)
-		{
-			if (n == pf->fd)
-			{
-				return pf;
-			}
-		}
+		FB_SIZE_T pos;
+		if (slct_poll.find(n, pos))
+			return &slct_poll[pos];
 
 		return NULL;
 	}
-
-	static int compare(const void* a, const void* b)
-	{
-		// use C-cast here to be for sure compatible with libc
-		return ((pollfd*) a)->fd - ((pollfd*) b)->fd;
-	}
 #endif
 
 public:
 #ifdef HAVE_POLL
 	Select()
-		: slct_time(0), slct_count(0), slct_poll(*getDefaultMemoryPool())
+		: slct_time(0), slct_count(0), slct_poll(*getDefaultMemoryPool()),
+		  slct_ready(*getDefaultMemoryPool())
 	{ }
 
 	explicit Select(Firebird::MemoryPool& pool)
-		: slct_time(0), slct_count(0), slct_poll(pool)
+		: slct_time(0), slct_count(0), slct_poll(pool), slct_ready(pool)
 	{ }
 #else
 	Select()
@@ -285,6 +275,40 @@ public:
 
 	enum HandleState {SEL_BAD, SEL_DISCONNECTED, SEL_NO_DATA, SEL_READY};
 
+	// set first port to check for readyness
+	void checkStart(RemPortPtr& port)
+	{
+		slct_main = port;
+		slct_port = port;
+	}
+
+	// get port to check for readyness
+	// assume port_mutex is locked
+	HandleState checkNext(RemPortPtr& port)
+	{
+		if (slct_port && slct_port->port_state == rem_port::DISCONNECTED)
+		{
+			// restart from main port
+			slct_port = NULL;
+			if (slct_main && slct_main->port_state == rem_port::DISCONNECTED)
+				slct_main = NULL;
+
+			slct_port = slct_main;
+		}
+
+		port = slct_port;
+		if (!slct_port)
+			return SEL_NO_DATA;
+
+#ifdef WIRE_COMPRESS_SUPPORT
+		if (slct_port->port_flags & PORT_z_data)
+			return SEL_READY;
+#endif
+
+		slct_port = slct_port->port_next;
+		return ok(port);
+	}
+
 	HandleState ok(const rem_port* port)
 	{
 #ifdef WIRE_COMPRESS_SUPPORT
@@ -295,9 +319,17 @@ public:
 #if defined(WIN_NT)
 		return FD_ISSET(n, &slct_fdset) ? SEL_READY : SEL_NO_DATA;
 #elif defined(HAVE_POLL)
-		const pollfd* pf = getPollFd(n);
+		pollfd* pf = NULL;
+		FB_SIZE_T pos;
+		if (slct_ready.find(n, pos))
+			pf = slct_ready[pos];
+
 		if (pf)
-			return pf->events & SEL_CHECK_MASK ? SEL_READY : SEL_NO_DATA;
+		{
+			HandleState ret = pf->events & SEL_CHECK_MASK ? SEL_READY : SEL_NO_DATA;
+			pf->events = 0;		// unset
+			return ret;
+		}
 		return n < 0 ? (port->port_flags & PORT_disconnect ? SEL_DISCONNECTED : SEL_BAD) : SEL_NO_DATA;
 #else
 		if (n < 0 || n >= FD_SETSIZE)
@@ -323,16 +355,18 @@ public:
 	void set(SOCKET handle)
 	{
 #ifdef HAVE_POLL
-		pollfd* pf = getPollFd(handle);
-		if (pf)
+		FB_SIZE_T pos;
+		if (slct_poll.find(handle, pos))
 		{
-			pf->events = SEL_INIT_EVENTS;
-			return;
+			slct_poll[pos].events = SEL_INIT_EVENTS;
+		}
+		else
+		{
+			pollfd f;
+			f.fd = handle;
+			f.events = SEL_INIT_EVENTS;
+			slct_poll.insert(pos, f);
 		}
-		pollfd f;
-		f.fd = handle;
-		f.events = SEL_INIT_EVENTS;
-		slct_poll.push(f);
 #else
 		FD_SET(handle, &slct_fdset);
 #ifdef WIN_NT
@@ -352,11 +386,14 @@ public:
 		slct_width = 0;
 		FD_ZERO(&slct_fdset);
 #endif
+		slct_main = NULL;
+		slct_port = NULL;
 	}
 
 	void select(timeval* timeout)
 	{
 #ifdef HAVE_POLL
+		slct_ready.clear();
 		bool hasRequest = false;
 		pollfd* const end = slct_poll.end();
 		for (pollfd* pf = slct_poll.begin(); pf < end; ++pf)
@@ -379,7 +416,11 @@ public:
 		if (slct_count >= 0)	// in case of error return revents may contain something bad
 		{
 			for (pollfd* pf = slct_poll.begin(); pf < end; ++pf)
+			{
 				pf->events = pf->revents;
+				if (pf->revents & SEL_CHECK_MASK)
+					slct_ready.add(pf);
+			}
 		}
 #else
 #ifdef WIN_NT
@@ -400,11 +441,21 @@ public:
 private:
 	int		slct_count;
 #ifdef HAVE_POLL
-	HalfStaticArray<pollfd, 8> slct_poll;
+	class PollToFD
+	{
+	public:
+		static int generate(const pollfd* p) { return p->fd; };
+		static int generate(const pollfd& p) { return p.fd; };
+	};
+
+	SortedArray<pollfd, InlineStorage<pollfd, 8>, int, PollToFD>  slct_poll;
+	SortedArray<pollfd*, InlineStorage<pollfd*, 8>, int, PollToFD>  slct_ready;
 #else
 	int		slct_width;
 	fd_set	slct_fdset;
 #endif
+	RemPortPtr slct_main;	// first port to check for readyness
+	RemPortPtr slct_port;	// next port to check for readyness
 };
 
 static bool		accept_connection(rem_port*, const P_CNCT*);
@@ -835,6 +886,7 @@ rem_port* INET_connect(const TEXT* name,
 	if (host.hasData())
 	{
 		delete port->port_connection;
+		port->port_connection = NULL;
 		port->port_connection = REMOTE_make_string(host.c_str());
 	}
 	else {
@@ -1731,7 +1783,7 @@ static void disconnect(rem_port* const port)
 	if (port->port_thread_guard && port->port_events_thread && !Thread::isCurrent(port->port_events_threadId))
 		port->port_thread_guard->setWait(port->port_events_thread);
 	else
-		port->release();
+		port->releasePort();
 
 #ifdef DEBUG
 	if (INET_trace & TRACE_summary)
@@ -2005,6 +2057,13 @@ static bool select_multi(rem_port* main_port, UCHAR* buffer, SSHORT bufsize, SSH
  *
  **************************************/
 
+// This code is used to test error handling in main server loop
+#ifdef NEVERDEF
+	static int dummyCnt = 0;
+	if (++dummyCnt % 64 == 0)
+		(Arg::Gds(isc_random) << "Simulated select_multi error").raise();
+#endif
+
 	for (;;)
 	{
 		select_port(main_port, &INET_select, port);
@@ -2115,11 +2174,11 @@ static void select_port(rem_port* main_port, Select* selct, RemPortPtr& port)
  **************************************/
 
 	MutexLockGuard guard(port_mutex, FB_FUNCTION);
-
-	for (port = main_port; port; port = port->port_next)
+	while (true)
 	{
-		Select::HandleState result = selct->ok(port);
-		selct->unset(port->port_handle);
+		Select::HandleState result = selct->checkNext(port);
+		if (!port)
+			return;
 
 		switch (result)
 		{
@@ -2283,6 +2342,9 @@ static bool select_wait( rem_port* main_port, Select* selct)
 
 			if (selct->getCount() != -1)
 			{
+				RemPortPtr p(main_port);
+				selct->checkStart(p);
+
 				// if selct->slct_count is zero it means that we timed out of
 				// select with nothing to read or accept, so clear the fd_set
 				// bit as this value is undefined on some platforms (eg. HP-UX),
@@ -2977,6 +3039,9 @@ static bool packet_receive(rem_port* port, UCHAR* buffer, SSHORT buffer_length,
 
 			if (!slct_count)
 			{
+				if (port->port_protocol == 0)
+					return false;
+
 #ifdef DEBUG
 				if (INET_trace & TRACE_operations)
 				{
@@ -2991,11 +3056,6 @@ static bool packet_receive(rem_port* port, UCHAR* buffer, SSHORT buffer_length,
 				}
 				continue;
 			}
-
-			if (!slct_count && port->port_protocol == 0)
-			{
-				return false;
-			}
 		}
 
 		n = recv(port->port_handle, reinterpret_cast<char*>(buffer), buffer_length, 0);
